import re
from datetime import datetime
##

from ..lib.driver import Driver
from ..util import defaults as DEFAULT
from ..util.settings import Settings
from .file import File, Folder
from .user import User

class Promotion:
    """Promotion class"""

    def __init__(self):
        """Promotion object"""

        # the amount to discount
        self.amount = None
        # the number of trials to allow
        self.limit = None
        # the expiration of the trial
        self.expiration = None
        # the duration of the discount
        self.duration = None
        # the user to apply the promotion to
        self.user = None
        # the message to provide with the promotion
        self.message = None
        # prevents double prompts
        self.gotten = False

    @staticmethod
    def apply_to_user():
        """Applies promotion directly to user via their profile page
        
           Applying a discount to a user requires:
           - amount
           - duration
           - expiration
           - message
           - user

        """

        print("Promotion - Apply To User")
        p = Promotion()
        # ensure the promotion has non default values, return early if missing
        # p.get()
        gotten = p.get_amount()
        if not gotten: return False
        gotten = p.get_duration()
        if not gotten: return False
        gotten = p.get_expiration()
        if not gotten: return False
        gotten = p.get_message()
        if not gotten: return False
        gotten = p.get_user()
        if not gotten: return False
        # prompt skip
        from .driver import Driver
        # get default driver and apply the promotion directly
        Driver.promotion_user_directly(promotion=p)
        return True

    @staticmethod
    def create_campaign():
        """Creates a Promotional Campaign

           A campaign consists of:
           - amount
           - duration
           - expiration
           - limit
           - user
           - text

        """

        print("Promotion - Creating Campaign")
        p = Promotion()
        # ensure the promotion has non default values, return early if missing
        # p.get()
        gotten = p.get_amount()
        if not gotten: return False
        gotten = p.get_user()
        if not gotten: return False
        gotten = p.get_expiration()
        if not gotten: return False
        gotten = p.get_limit()
        if not gotten: return False
        gotten = p.get_duration()
        if not gotten: return False
        gotten = p.get_message()
        if not gotten: return False
        # prompt skip
        from .driver import Driver
        # get the default driver and enter the promotion campaign
        Driver.promotional_campaign(promotion=p)
        return True

    # requires the copy/paste and email steps
    @staticmethod
    def create_trial_link():
        """Creates a Promotional Trial Link

           A trial link consists of:
           - duration
           - expiration
           - limit
           - message
           - user
            
           Note: this creates a free trial link but does NOT send it to the user
           because it is incomplete. The copy/paste step to message to a user is nonfunctioning.           

        """

        print("Promotion - Creating Trial Link")
        p = Promotion()
        # ensure the promotion has non default values, return early if missing
        # p.get()
        gotten = p.get_duration()
        if not gotten: return False
        gotten = p.get_expiration()
        if not gotten: return False
        gotten = p.get_limit()
        if not gotten: return False
        gotten = p.get_message()
        if not gotten: return False
        gotten = p.get_user()
        if not gotten: return False
        # if not self.gotten: return
        # limit, expiration, months, user
        from .driver import Driver
        link = Driver.promotional_trial_link(promotion=p)
        # text = "Here's your free trial link!\n"+link
        # Settings.dev_print("Link: "+str(text))
        # Settings.send_email(email, text)
        return True

    def get(self):
        """Update the promotion object's default values"""

        return dict({
            "user": self.get_user(),
            "amount": self.get_amount(),
            "expiration": self.get_expiration(),
            "limit": self.get_limit(),
            "duraction": self.get_duration(),
            "message": self.get_message()
        })

    def get_amount(self):
        """
        Gets the amount value if not none else sets it from args or prompts.

        Returns
        -------
        int
            The amount as an int

        """

        if self.amount: return self.amount
        # retrieve from args and return if exists
        amount = Settings.get_amount() or None
        if amount: 
            self.amount = amount
            return amount
        # prompt skip
        if not Settings.confirm(amount): return self.get_amount()
        self.amount = amount
        return self.amount

    def get_expiration(self):
        """
        Gets the expiration value if not none else sets it from args or prompts.

        Returns
        -------
        int
            The expiration as an int

        """

        if self.expiration: return self.expiration
        # retrieve from args and return if exists
        expiration = Settings.get_expiration() or None
        if expiration: 
            self.expiration = expiration
            return expiration
        # prompt skip
        # confirm expiration
        if not Settings.confirm(expiration): return self.get_expiration()
        self.expiration = expiration
        return self.expiration

    def get_limit(self):
        """
        Gets the expiration value if not none else sets it from args or prompts.

        Returns
        -------
        int
            The expiration as an int

        """

        if self.limit: return self.limit
        # retrieve from args and return if exists
        limit = Settings.get_promotion_limit() or None
        if limit: 
            self.limit = limit
            return limit
        # prompt skip
        # confirm limit
        if not Settings.confirm(limit): return self.get_limit()
        self.limit = limit
        return self.limit

    def get_message(self):
        """
        Gets the message value if not none else sets it from args or prompts.

        Returns
        -------
        str
            The message as a str

        """

        if self.message != None: return self.message
        # retrieve from args and return if exists
        message = Settings.get_text() or None
        if message: 
            self.message = message
            return message
        # prompt skip
        # confirm message
        if not Settings.confirm(message): return self.get_text()
        self.message = message
        return self.message

    def get_duration(self):
        """
        Gets the duration value if not none else sets it from args or prompts.

        Returns
        -------
        int
            The duration as an int

        """

        if self.duration: return self.duration
        # retrieve from args and return if exists
        duration = Settings.get_promo_duration() or None
        if duration: 
            self.duration = duration
            return duration
        # duration skip
        # confirm duration
        if not Settings.confirm(duration): return self.get_duration()
        self.duration = duration
        return self.duration

    def get_user(self):
        """
        Populate and get the username value

        If not found in args and prompt is enabled, ask for value.

        Returns
        -------
        User
            the user to apply the promotion to

        """

        if self.user: return self.user
        user = User.select_user()
        self.user = user.username
        return self.user

    @staticmethod
    def grandfathered():
        """
        Executes the 'Grandfather' promotion model

        In groups of 5, existing users will be added to the 'Grandfathered' OnlyFans list and
        then provided with the max discount for the max months. If the process interrupts, 
        running again will continue to discount users not yet added to the list.

        """

        print("Promotion - Grandfather")
        # prompt skip
        Settings.maybe_print("getting users to grandfather")
        # get all users
        users = User.get_all_users()
        from .driver import Driver
        # get all users from logged in user's 'grandfathered' list
        users_, name, number = Driver.get_list(name="grandfathered")
        # remove all users that have already been grandfathered from the list of all users
        # users = [user for user in users if user not in users_] # i guess doesn't work?
        for i, user in enumerate(users[:]):
            for user_ in users_:
                for key, value in user_.items():
                    if str(key) == "username" and str(user.username) == str(value):
                        users.remove(user)

        def chunks(lst, n):
            """Yield successive n-sized chunks from lst."""
            for i in range(0, len(lst), n):
                yield lst[i:i + n]

        # get users in groups of 5 to allow performance over interrupts
        userChunks = chunks(users, 5)
        num = 1
        for userChunk in userChunks:
            print("Chunk: {}/{}".format(num, len(users)/5))
            num += 1
            # add users to 'grandfathered' list prior to discounting
            Settings.maybe_print("grandfathering: {}".format(len(userChunk)))
            try:
                successful = Driver.add_users_to_list(users=userChunk, number=number, name="grandfathered")
                # if successful then discount
                if not successful: return
                d = Discount() # discount will fill defaults with promotion values
                d.grandfatherer(users=userChunk)
            except Exception as e:
                Settings.dev_print(e)
        return True



import os
from datetime import datetime
from pathlib import Path

##
# Defaults 
##

ACTIONS = [ "Discount", "Message", "Post", "Profile", "Promotion" ]

AMOUNT_NONE = 0

DATE_FORMAT = "%m/%d/%Y"
TIME_FORMAT = "%H:%M:%S"
SCHEDULE_FORMAT = "{} {}".format(DATE_FORMAT, TIME_FORMAT)

date_ = datetime.strptime(str(datetime.now().strftime(SCHEDULE_FORMAT)), SCHEDULE_FORMAT)

DATE = date_.date().strftime(DATE_FORMAT)
TIME = date_.time().strftime(TIME_FORMAT)
SCHEDULE = date_.strftime(SCHEDULE_FORMAT)

TIME_NONE = "00:00:00"

DEFAULT_MESSAGE = ":)"
DEFAULT_REFRESHER = "hi!"
DEFAULT_GREETING = "hi! thanks for subscribing :3 do you have any preferences?"

DISCOUNT_MAX_AMOUNT = 55
DISCOUNT_MIN_AMOUNT = 5
DISCOUNT_MAX_MONTHS = 12
DISCOUNT_MIN_MONTHS = 1

## note: '99' aka 'No Limit' no longer allowed?
# DURATION_ALLOWED = [1,3,7,30, 99] # in days
DURATION_ALLOWED = [1,3,7,30] # in days
DURATION_NONE = 0

EXPIRATION_MIN = 1
EXPIRATION_MAX = 30
EXPIRATION_NONE = 0

IMAGE_LIMIT = 15

LIMIT_ALLOWED = [0,1,2,3,4,5,6,7,8,9,10,20,30,40,50,60,70,80,90,100] # in %

MESSAGE_CHOICES = ["all", "recent", "favorite", "renew on"]

PROMOTION_EXPIRATION_ALLOWED = [int(i) for i in range(30)] # in %
PROMOTION_EXPIRATION_ALLOWED.insert(0,0)
# number
PROMOTION_OFFER_LIMIT = [0,1,2,3,4,5,6,7,8,9,10]
# various datetime
PROMOTION_DURATION_ALLOWED = ["1 day","3 days","7 days","14 days","1 month","3 months","6 months","12 months"]

PRICE_MINIMUM = 3
PRICE_MAXIMUM = 200

# sftp
# selenium browser
REMOTE_BROWSER = "127.0.0.1"
BROWSER_PORT = 4444

REMOTE_HOST = "127.0.0.1"
REMOTE_PORT = 22

#3600 # 1hr in seconds
UPLOAD_MAX_DURATION = 60 # 1 minute


USER_LIMIT = 10

#########
# Paths #
#########

import getpass
USER = getpass.getuser()
# USER = os.getenv('USER')
if str(os.getenv('SUDO_USER')) != "root" and str(os.getenv('SUDO_USER')) != "None":
    USER = os.getenv('SUDO_USER')

# linux default
HOME_DIR = "/home"
# check for Windows
if os.name == 'nt':
    HOME_DIR = "C:\\Users"
ROOT_PATH = os.path.join(HOME_DIR, USER, ".onlysnarf")
    
DOWNLOAD_PATH = os.path.join(ROOT_PATH, "downloads")
UPLOAD_PATH = os.path.join(ROOT_PATH, "uploads")
LOG_PATH = os.path.join(ROOT_PATH, "snarf.log")
REMOTE_PATH = ROOT_PATH
CONFIGS_PATH = os.path.join(ROOT_PATH, "conf")
USERS_PATH = os.path.join(CONFIGS_PATH, "users.json")
PROFILE_PATH = os.path.join(CONFIGS_PATH, "profile.json")
CONFIG_PATH = os.path.join(CONFIGS_PATH, "config.conf")
if os.environ.get('ENV') == "test":
    # CONFIG_PATH = os.path.join(CONFIGS_PATH, "test-config.conf")
    CONFIG_PATH = os.path.join(os.getcwd(), "OnlySnarf", "conf", "test-config.conf")
PROFILES_PATH = os.path.join(CONFIGS_PATH, "users")

if os.environ.get('ENV') == "test":
    print("Paths:")
    print("root: "+ROOT_PATH)
    print("configs: "+CONFIGS_PATH)
    print("config: "+CONFIG_PATH)
    print("download: "+DOWNLOAD_PATH)
    print("log: "+LOG_PATH)
    print("profiles: "+PROFILES_PATH)
    print("profile: "+PROFILE_PATH)
    print("remote: "+REMOTE_PATH)
    print("upload: "+UPLOAD_PATH)
    print("users: "+USERS_PATH)

Path(ROOT_PATH).mkdir(parents=True, exist_ok=True)
Path(DOWNLOAD_PATH).mkdir(parents=True, exist_ok=True)
Path(UPLOAD_PATH).mkdir(parents=True, exist_ok=True)
Path(CONFIGS_PATH).mkdir(parents=True, exist_ok=True)



import os
os.environ['ENV'] = "test"
import unittest

from OnlySnarf.util.config import config
# from OnlySnarf.util import defaults as DEFAULT
from OnlySnarf.lib.driver import Driver
from OnlySnarf.util.settings import Settings
from OnlySnarf.snarf import Snarf
# from OnlySnarf.classes.user import User

class TestPromotion(unittest.TestCase):

    def setUp(self):
        Settings.set_debug("tests")
        config["prefer_local"] = False
        self.test_snarf = Snarf()

    def tearDown(self):
        self.test_snarf.close()

    @unittest.skip("todo")
    def test_promotion_campaign(self):
        config["promotion_method"] = "campaign"
        assert self.test_snarf.promotion(), "unable to apply promotion: campaign"

    @unittest.skip("todo")
    def test_promotion_trial(self):
        config["promotion_method"] = "trial"
        assert self.test_snarf.promotion(), "unable to apply promotion: trial"

    @unittest.skip("todo")
    def test_promotion_user(self):
        config["promotion_method"] = "user"
        assert self.test_snarf.promotion(), "unable to apply promotion: user"

    @unittest.skip("todo")
    def test_promotion_grandfather(self):
        config["promotion_method"] = "grandfather"
        assert self.test_snarf.promotion(), "unable to apply promotion: grandfather"

############################################################################################

if __name__ == '__main__':
    unittest.main()


import ffmpeg
import datetime
import os
##
from ..util.settings import Settings

##################
##### FFMPEG #####
##################

# all videos in folder into single mp4
def combine(folderPath):
    # if str(Settings.COMBINE) == "False":
    #     Settings.warn_print("skipping combine")
    #     return False
    if ".mp4" not in str(folderPath):
        Settings.err_print("unable to combine")
        return False
    combinePath = str(folderPath).replace(".mp4", "_full.mp4")
    try:    
        ffmpeg.input(str(folderPath), format='concat', safe=0).output(combinePath, c='copy').run()
    except Exception as e:
        Settings.dev_print(e)
        if "Conversion failed!" in str(e):
            Settings.err_print("combine failure")
            return combinePath                    
    Settings.print("Combine Complete")
    return combinePath


# images from gallery
def gifify(path):
    # First convert the images to a video:
    # ffmpeg -f image2 -i image%d.jpg video.avi
    loglevel = "quiet"
    # if Settings.is_debug():
        # loglevel = "debug"
    # p = subprocess.call(['ffmpeg', '-loglevel', str(loglevel), '-y', '-i', str(path), '-c', 'copy', '-c:v', 'libx264', '-c:a', 'aac', '-strict', '2', '-crf', '26', '-b:v', str(bitrate), str(reducedPath)])
    # Then convert the avi to a gif:
    # ffmpeg -i video.avi -pix_fmt rgb24 -loop_output 0 out.gif

# frames for preview gallery
def frames(path):
    try:
        Settings.maybe_print("capturing frames: {}".format(path))
        try:
            clip = VideoFileClip(str(path))
            Settings.maybe_print("length: {}".format(clip.duration))
        except FileNotFoundError:
            Settings.err_print("missing file to capture frames")
            return path
    except: pass
    screenshots = []
    # ffmpeg -i test.avi -vcodec png -ss 10 -vframes 1 -an -f rawvideo test.png
    for i in range(10):
        output = path.replace(".mp4", "-{}.png".format(i))
        p = subprocess.call(['ffmpeg', '-loglevel', str(loglevel), '-y', '-ss', str(int(i)*10), '-i', str(path), '-vcodec', 'png', '-vframes', '1', '-an', '-f', 'rawvideo', output])
        screenshots.append(output)
        if int(i)*10 > int(clip.duration): break
    return screenshots

# this requires a similar video and has no real use so remove?
# or change to some other repair method for videos
# def repair(path):
#     if str(Settings.get_repair()) == "False":
#         Settings.warn_print("skipping repair")
#         return path
#     if ".mp4" not in str(path):
#         Settings.err_print("unable to repair")
#         return path
#     if not os.path.isfile(str(Settings.WORKING_VIDEO)):
#         Settings.err_print("missing working video")
#         return path
#     repairedPath = str(path).replace(".mp4", "_fixed.mp4")
#     try:
#         Settings.print("Repairing: {} <-> {}".format(path, Settings.WORKING_VIDEO))
#         if Settings.is_debug():
#             subprocess.call(['untrunc', str(Settings.WORKING_VIDEO), str(path)]).communicate()
#         else:
#             subprocess.Popen(['untrunc', str(Settings.WORKING_VIDEO), str(path)],stdin=FNULL,stdout=FNULL)
#     except AttributeError:
#         if os.path.isfile(str(path)+"_fixed.mp4"):
#             shutil.move(str(path)+"_fixed.mp4", repairedPath)
#             Settings.print("Repair Complete")
#     except:
#         Settings.maybe_print(sys.exc_info()[0])
#         Settings.warn_print("skipping repair")
#         return path
#     Settings.print("Repair Successful")
#     return str(repairedPath)

def reduce(path):
    if not Settings.is_reduce():
        Settings.warn_print("skipping reduction")
        return path
    if ".mp4" not in str(path):
        Settings.err_print("unable to reduce")
        return path
    reducedPath = str(path).replace(".mp4", "_reduced.mp4")
    try:
        Settings.maybe_print("reducing: {}".format(path))
        try:
            clip = VideoFileClip(str(path))
            Settings.maybe_print("length: {}".format(clip.duration))
            bitrate = 1000000000 / int(clip.duration)
            Settings.maybe_print("bitrate: {}".format(bitrate))
        except FileNotFoundError:
            Settings.err_print("missing file to reduce")
            return path
        loglevel = "quiet"
        if Settings.is_debug():
            loglevel = "debug"
        p = subprocess.call(['ffmpeg', '-loglevel', str(loglevel), '-err_detect', 'ignore_err', '-y', '-i', str(path), '-c', 'copy', '-c:v', 'libx264', '-c:a', 'aac', '-strict', '2', '-crf', '26', '-b:v', str(bitrate), str(reducedPath)])
        # p.communicate()
    except FileNotFoundError:
        Settings.warn_print("ignoring fixed video")
        return reduce(str(path).replace(".mp4", "_fixed.mp4"))
    except Exception as e:
        Settings.dev_print(e)
        if "Conversion failed!" in str(e):
            Settings.err_print("conversion failure")
            return path                    
    Settings.print("Reduction Complete")
    originalSize = os.path.getsize(str(path))
    newSize = os.path.getsize(str(reducedPath))
    Settings.print("Original Size: {}kb - {}mb".format(originalSize/1000, originalSize/1000000))
    Settings.print("Reduced Size: {}kb - {}mb".format(newSize/1000, newSize/1000000))
    if int(originalSize) < int(newSize):
        Settings.warn_print("original size smaller")
        return path
    if int(newSize) == 0:
        Settings.err_print("missing reduced file")
        return path
    return reducedPath


# into segments (60 sec, 5 min, 10 min)
# segment: minutes
def split(path, segment):
    if not Settings.is_split():
        Settings.warn_print("skipping split")
        return path
    if ".mp4" not in str(path):
        Settings.err_print("unable to split")
        return path
    splitPaths = []
    splitPath = str(path).replace(".mp4", "_split$.mp4")
    try:
        Settings.maybe_print("splitting: {}".format(path))
        try:
            clip = VideoFileClip(str(path))
            Settings.maybe_print("length: {}".format(clip.duration))
        except FileNotFoundError:
            Settings.err_print("missing file to split")
            return path
        i = 0
        index = 0
        loglevel = "quiet"
        if Settings.is_debug():
            loglevel = "debug"
        while True:
            start = datetime.timedelta(seconds=index)
            end = datetime.timedelta(seconds=int(index)+(60*int(segment)))
            out = splitPath.replace("$",i)
            p = subprocess.call(['ffmpeg', '-loglevel', str(loglevel), '-ss', str(start), '-y', '-i', str(path), '-to', str(end), '-c', 'copy', '-c:v', 'libx264', '-c:a', 'aac', '-strict', '2', str(out)])
            splitPaths.append(out)
            index += 60*int(segment)
            i += 1
        # p.communicate()
    except Exception as e:
        Settings.dev_print(e)
        if "Conversion failed!" in str(e):
            Settings.err_print("split failure")
            return splitPaths                    
    Settings.print("Split Complete")
    return splitPaths

def thumbnail_fix(path):
    # if str(Settings.THUMBNAILING_PREVIEW) == "False":
    #     Settings.warn_print("preview thumbnailing disabled")
    #     return path
    try:
        Settings.print("Thumbnailing: {}".format(path))
        loglevel = "quiet"
        if Settings.is_debug():
            loglevel = "debug"
        thumbnail_path = os.path.join(os.path.dirname(str(path)), 'thumbnail.png')
        Settings.maybe_print("thumbnail path: {}".format(thumbnail_path))
        p = subprocess.call(['ffmpeg', '-loglevel', str(loglevel), '-i', str(path),'-ss', '00:00:00.000', '-vframes', '1', str(thumbnail_path)])
        p.communicate()
        Settings.print("Thumbnailing Complete")
        return thumbedPath
    except FileNotFoundError:
        Settings.warn_print("ignoring thumbnail")
    except AttributeError:
        Settings.print("Thumbnailing: Captured PNG")
    except:
        Settings.maybe_print(sys.exc_info()[0])
        Settings.err_print("thumbnailing fuckup")    

#seconds off front or back
def trim(path):
    if not Settings.is_trim():
        Settings.warn_print("skipping trim")
        return path
    if ".mp4" not in str(path):
        Settings.err_print("unable to trim")
        return path
    reducedPath = str(path).replace(".mp4", "_trimmed.mp4")
    try:
        Settings.maybe_print("trimming: {}".format(path))
        try:
            clip = VideoFileClip(str(path))
            Settings.maybe_print("length: {}".format(clip.duration))
        except FileNotFoundError:
            Settings.err_print("missing file to reduce")
            return path
        start = datetime.timedelta(seconds=60)
        end = datetime.timedelta(seconds=clip.duration-60)
        loglevel = "quiet"
        if Settings.is_debug():
            loglevel = "debug"
        p = subprocess.call(['ffmpeg', '-loglevel', str(loglevel), '-ss', str(start), '-y', '-i', str(path), '-to', str(end), '-c', 'copy', '-c:v', 'libx264', '-c:a', 'aac', '-strict', '2', str(reducedPath)])
        # p.communicate()
    except Exception as e:
        Settings.dev_print(e)
        if "Conversion failed!" in str(e):
            Settings.err_print("trim failure")
            return path                    
    Settings.print("Trim Complete")
    return reducedPath

# unnecessary, handled by onlyfans
def watermark():
    pass
# cleanup & label appropriately (digital watermarking?)
def metadata():
    pass



import argparse
from .validators import valid_amount, valid_price, valid_path
from .validators import valid_date, valid_month, valid_schedule, valid_time
from .validators import valid_duration, valid_expiration, valid_limit
from .validators import valid_promo_duration, valid_promo_expiration
from . import defaults as DEFAULT

def apply_args(parser):

  #############
  ## General ##
  #############

  ##
  # -browser
  parser.add_argument('-browser', '-B', type=str, default="auto", choices=["auto","brave","chrome","chromium","firefox","remote"], dest='browser', help='web browser to use')
  # parser.add_argument('-browser', '-B', type=str, default="auto", choices=["auto","brave","chrome","chromium","firefox","ie","edge","opera","remote"], dest='browser', help='web browser to use')
  ##
  # -login
  # method to prefer when logging in
  # note: "google" is disabled due to updates&testing requirements
  parser.add_argument('-login', '-L', dest='login', default="auto", choices=["auto","onlyfans","twitter"], help='method of user login to prefer')
  ##
  # -reduce
  # enables file reduction
  parser.add_argument('-reduce', action='store_true', dest='reduce', help='enable reducing files over 50 MB')
  ##
  # --save
  # saves OnlyFans users upon exit
  parser.add_argument('-save', '-S', action='store_true', dest='save_users', help='enable saving users locally on exit')
  ##
  # -tweet
  # enabled tweeting
  parser.add_argument('-tweet', action='store_true', dest='tweeting', help='enable tweeting when posting')
  ##
  # --username
  # OnlyFans username to use
  parser.add_argument('--username', '--u', type=str, default="", dest='username', help='OnlyFans username to use')
  ##
  # -phone
  # OnlyFans phone number to use for additional login steps
  parser.add_argument('-phone', type=str, default="", dest='phone', help='OnlyFans phone number to use')

  ###############
  ## DEBUGGING ##
  ###############

  ##
  # -config
  # path to config.conf file
  parser.add_argument('-config', '-C', dest="path_config", type=str, help='path to config.conf', default=DEFAULT.CONFIG_PATH)
  ##
  # -cookies
  # load & save from/to local cookies path
  parser.add_argument('-cookies', action='store_true', dest='cookies', help=argparse.SUPPRESS)
  # -debug
  # debugging - skips uploading and deleting unless otherwise forced
  parser.add_argument('-debug', '-D', action='store_true', dest='debug', help='enable debugging')
  ##
  # -debug-delay
  # user message delay
  parser.add_argument('-debug-delay', action='store_true', dest='debug_delay', help=argparse.SUPPRESS)
  ##
  # -debug-firefox
  # enables trace logging for firefox
  parser.add_argument('-debug-firefox', action='store_true', dest='debug_firefox', help=argparse.SUPPRESS)
  ##
  # -debug-google
  # enables trace logging for google chrome
  parser.add_argument('-debug-chrome', action='store_true', dest='debug_chrome', help=argparse.SUPPRESS)
  ##
  # -debug-selenium
  # enables selenium logging
  parser.add_argument('-debug-selenium', action='store_true', dest='debug_selenium', help=argparse.SUPPRESS)
  ##
  # -download-max
  # maximum number of images to download
  parser.add_argument('-download-max', type=int, default=DEFAULT.IMAGE_LIMIT, dest="download_limit", help=argparse.SUPPRESS)
  ##
  # -download-path
  # the path to the downloaded files
  parser.add_argument('-download-path', type=str, dest='path_download', default=DEFAULT.DOWNLOAD_PATH, help=argparse.SUPPRESS)
  ##
  # -force-upload
  # ignore upload max wait
  parser.add_argument('-force-upload', action='store_true', dest='force_upload', help=argparse.SUPPRESS)
  ##
  # -keep
  # keep the browser window open
  parser.add_argument('-keep', '-K', action='store_true', dest='keep', help='keep browser window open after scripting ends')
  ##
  # -prefer-local
  # prefers local user cache over refreshing first call
  parser.add_argument('-prefer-local', default=True, action='store_false', dest='prefer_local', help='prefer recently cached data')
  ##
  # -show
  # shows window
  parser.add_argument('-show', '-SW', dest='show', action='store_true',  help='enable displaying browser window')
  ##
  # -skip-download
  parser.add_argument('-skip-download', action='store_true', dest='skip_download', help=argparse.SUPPRESS)
  ##
  # -skip-upload
  # skips file upload
  parser.add_argument('-skip-upload', action='store_true', dest='skip_upload', help=argparse.SUPPRESS)
  ##
  # -skip-users
  # list of users to skip
  parser.add_argument('-skip-users', dest='skipped_users', action='append', help=argparse.SUPPRESS)
  ##
  # -upload-max
  # maximum number of images that can be uploaded
  parser.add_argument('-upload-max', type=int, default=DEFAULT.IMAGE_LIMIT, dest='upload_max', help=argparse.SUPPRESS)
  ##
  # -upload-max-duration
  # the max number of 10 minute intervals to upload for
  parser.add_argument('-upload-max-duration', dest='upload_max_duration', default=DEFAULT.UPLOAD_MAX_DURATION, type=int, help=argparse.SUPPRESS)
  ##
  # -user-path
  # the path to the users.json file
  parser.add_argument('-users-path', type=str, dest='path_users', default=DEFAULT.USERS_PATH, help=argparse.SUPPRESS)
  ##
  # -verbose
  # v, vv, vvv
  parser.add_argument('-v', '-verbose', dest="verbose", action='count', default=0, help="verbosity level (max 3)")


def apply_subcommand_args(parser):

  subparsers = parser.add_subparsers(help='Include a sub-command to run a corresponding action:', dest="action", required=True)

  ##############
  ## Discount ##
  ##############

  parser_discount = subparsers.add_parser('discount', help='> discount one or more users')
  userAndUsers = parser_discount.add_mutually_exclusive_group()
  ##
  # -amount
  # action: discount
  # the amount to discount a user by
  parser_discount.add_argument('-amount', type=valid_amount, dest='amount', help='amount (%%) to discount by', default=DEFAULT.AMOUNT_NONE)
  ##
  # -months
  # action: discount
  # the number of months to discount for
  parser_discount.add_argument('-months', type=valid_month, default=None, dest='months', help='number of months to discount')
  ##
  # -user
  # the user to discount
  userAndUsers.add_argument('-user', type=str,  default=None, dest='user', help='user to discount')
  ##
  # -users
  # the users to discount
  userAndUsers.add_argument('-users', dest='users', action='append', default=[], help='users to discount')

  ##########
  ## Menu ##
  ##########

  parser_menu = subparsers.add_parser('menu', help='> access the cli menu')

  #############
  ## Message ##
  #############

  parser_message = subparsers.add_parser('message', help='> send a message to one or more users')
  dateAndSchedule = parser_message.add_mutually_exclusive_group()
  userAndUsers = parser_message.add_mutually_exclusive_group()
  ##
  # -date
  # date in MM-DD-YYYY
  dateAndSchedule.add_argument('-date', type=valid_date, default=DEFAULT.DATE, dest='date', help='schedule date (MM-DD-YYYY)')
  ##
  # -performers
  # list of performers to tag
  parser_message.add_argument('-performers', dest='performers', action='append',  default=[], help='performers to reference. adds \"@[...performers]\"')
  ##
  # -price
  # the price to be set in a message
  parser_message.add_argument('-price', type=valid_price, help='price to charge ($)', default=0, dest='price')
  ##
  # -schedule
  # the schedule to upload a post for
  dateAndSchedule.add_argument('-schedule', type=valid_schedule, default=DEFAULT.SCHEDULE, dest='schedule', help='schedule (MM-DD-YYYY:HH:MM:SS)')
  ##
  # -time
  # time in HH:MM
  parser_message.add_argument('-time', type=valid_time, default=DEFAULT.TIME, dest='time', help='time (HH:MM)')
  ##
  # -tags
  # @[tag]
  parser_message.add_argument('-tags', dest='tags', action='append', default=[], help='the tags (@[tag])')
  ##
  # -text
  # text for message or upload
  parser_message.add_argument('-text', default=None, dest='text', help='text to send')
  ##
  # -user
  # the user to message
  userAndUsers.add_argument('-user', type=str,  default=None, dest='user', help='user to message')
  ##
  # -users
  # the users to message
  userAndUsers.add_argument('-users', dest='users', action='append', default=[], help='users to message')
  ##
  # input
  parser_message.add_argument('input', default=[], nargs=argparse.REMAINDER, type=valid_path, help='one or more paths to files (or folder) to include in the message')

  ##########
  ## Post ##
  ##########

  parser_post = subparsers.add_parser('post', help='> upload a post')
  dateAndSchedule = parser_post.add_mutually_exclusive_group()
  durationAndExpiration = parser_post.add_mutually_exclusive_group()

  ##
  # -date
  # date in MM-DD-YYYY
  dateAndSchedule.add_argument('-date', type=valid_date, default=DEFAULT.DATE, dest='date', help='schedule date (MM-DD-YYYY)')
  ##
  # -duration
  # poll duration
  durationAndExpiration.add_argument('-duration', type=valid_duration, dest='duration',
    help='duration in days (99 for \'No Limit\') for a poll', choices=DEFAULT.DURATION_ALLOWED, default=DEFAULT.DURATION_NONE)
  ##
  # -expiration
  # date of post or poll expiration
  durationAndExpiration.add_argument('-expiration', type=valid_expiration, dest='expiration', help='expiration in days (999 for \'No Limit\')', default=DEFAULT.EXPIRATION_NONE)
  ##
  # -performers
  # list of performers to tag
  parser_post.add_argument('-performers', dest='performers', action='append',  default=[], help='performers to reference. adds \"@[...performers]\"')
  ##
  # -price
  # price to be set in a message
  parser_post.add_argument('-price', type=valid_price, help='price to charge ($)', default=0, dest='price')
  ##
  # -schedule
  # schedule to upload a post for
  dateAndSchedule.add_argument('-schedule', type=valid_schedule, default=DEFAULT.SCHEDULE, dest='schedule', help='schedule (MM-DD-YYYY:HH:MM:SS)')
  ##
  # -time
  # time in HH:MM
  parser_post.add_argument('-time', type=valid_time, default=DEFAULT.TIME, dest='time', help='time (HH:MM)')
  ##
  # -tags
  # @[tag]
  parser_post.add_argument('-tags', dest='tags', action='append', default=[], help='tags (@[tag])')
  ##
  # -text
  # text for message or upload
  parser_post.add_argument('-text', default=None, dest='text', help='text to send')
  ##
  # -question
  # poll questions
  parser_post.add_argument('-question', '-Q', dest='questions', action='append', default=[], help='questions to ask')
  ##
  # input
  parser_post.add_argument('input', default=[], nargs=argparse.REMAINDER, type=valid_path, help='one or more paths to files (or folders) to include in the post')

  ##########
  ## User ##
  ##########

  parser_user = subparsers.add_parser('users', help='> scan & save users')

def apply_shim_args(parser):
  parser.add_argument('-amount', type=valid_amount, dest='amount', help='amount (%%) to discount by', default=DEFAULT.AMOUNT_NONE)
  parser.add_argument('-months', type=valid_month, default=None, dest='months', help='number of months to discount')
  parser.add_argument('-user', type=str,  default=None, dest='user', help='user to discount')
  parser.add_argument('-users', dest='users', action='append', default=[], help='users to discount')
  parser.add_argument('-date', type=valid_date, default=DEFAULT.DATE, dest='date', help='schedule date (MM-DD-YYYY)')
  parser.add_argument('-price', type=valid_price, help='price to charge ($)', default=0, dest='price')
  parser.add_argument('-schedule', type=valid_schedule, default=DEFAULT.SCHEDULE, dest='schedule', help='schedule (MM-DD-YYYY:HH:MM:SS)')
  parser.add_argument('-time', type=valid_time, default=DEFAULT.TIME, dest='time', help='time (HH:MM)')
  parser.add_argument('-tags', dest='tags', action='append', default=[], help='the tags (@[tag])')
  parser.add_argument('-text', default=None, dest='text', help='text to send')
  parser.add_argument('-duration', type=valid_duration, dest='duration', help='duration in days (99 for \'No Limit\') for a poll', choices=DEFAULT.DURATION_ALLOWED, default=DEFAULT.DURATION_NONE)
  parser.add_argument('-expiration', type=valid_expiration, dest='expiration', help='expiration in days (999 for \'No Limit\')', default=DEFAULT.EXPIRATION_NONE)
  parser.add_argument('-question', '-Q', dest='questions', action='append', default=[], help='questions to ask')
  parser.add_argument('input', default=[], nargs=argparse.REMAINDER, type=valid_path, help='one or more paths to files (or folder) to include in the message')



import re
from datetime import datetime
from ..lib.driver import Driver
from ..util.settings import Settings
from .user import User
##
from .file import File, Folder

class Poll:
    """OnlyFans Poll class"""

    def __init__(self):
        """OnlyFans Poll object"""

        # duration of poll
        self.duration = None
        # list of strings
        self.questions = []
        # prevents double prompts
        self.gotten = False

    def get(self):
        """
        Get the poll's values in a dict.

        Returns
        -------
        dict
            A dict containing the values of the poll

        """

        return dict({
            "duration": self.get_duration(),
            "questions": self.get_questions()
        })

    def get_duration(self):
        """
        Gets the duration value if not none else sets it from args or prompts.

        Returns
        -------
        int
            The duration as an int

        """

        if self.duration: return self.duration
        self.duration = Settings.get_duration()
        if int(self.duration) > 30: self.duration = "No limit"
        return self.duration

    def get_questions(self):
        """
        Gets the questions value if not none else sets it from args or prompts.

        Returns
        -------
        list
            The questions as strings in a list

        """

        if len(self.questions) > 0: return self.questions
        self.questions = Settings.get_questions()
        return self.questions

    def validate(self):
        """
        Determines whether or not the poll settings are valid.

        Returns
        -------
        bool
            Whether or not the poll is valid

        """

        Settings.dev_print("validating poll...")
        if len(self.get_questions()) > 0 and str(self.get_duration()) != "0":
            Settings.dev_print("valid poll!")
            return True
        Settings.dev_print("invalid poll!")
        return False


import json
from flask import Flask, request

from .util.config import config
from .util.settings import Settings
from .snarf import Snarf

def create_app():
    app = Flask(__name__)

    @app.route('/message', methods=['POST'])
    def message():
        try:
            return "", 200
        finally:
            args = json.loads(request.data)
            Settings.dev_print(args)
            config["text"] = args["text"]
            config["user"] = args["user"]
            try: config["files"] = args["input"].split(",")
            except Exception as e: pass
            try: config["price"] = args["price"] or 0
            except Exception as e: pass
            try: config["schedule"] = args["schedule"]
            except Exception as e: pass
            try: config["performers"] = args["performers"]
            except Exception as e: pass
            if app.testing:
                config["debug"] = True
                config["verbose"] = 3
            Snarf.message()
            Snarf.close()

    @app.route('/post', methods=['POST'])
    def post():
        try:
            return "", 200
        finally:
            args = json.loads(request.data)
            Settings.dev_print(args)
            config["text"] = args["text"]
            try: config["files"] = args["input"].split(",")
            except Exception as e: pass
            try: config["performers"] = args["performers"]
            except Exception as e: pass
            try: config["schedule"] = args["schedule"]
            except Exception as e: pass
            try: config["questions"] = args["questions"]
            except Exception as e: pass
            try: config["duration"] = args["duration"]
            except Exception as e: pass
            try: config["expires"] = args["expires"]
            except Exception as e: pass
            if app.testing:
                config["debug"] = True
                config["verbose"] = 3
            Snarf.post()
            Snarf.close()

    return app

def main():
    app = create_app()
    app.debug = True
    app.testing = True
    app.run(host="0.0.0.0", port=5000)

if __name__ == "__main__":
    main()


import json
import time
import os
import random
import threading
from datetime import datetime, timedelta
##
from ..util.colorize import colorize
from ..lib.driver import Driver
from ..util.settings import Settings

ALREADY_RANDOMIZED_USERS = []

class User:
    """OnlyFans users."""

    def __init__(self, data):
        """User object"""

        data = json.loads(json.dumps(data))
        self.name               =   data.get('name')                            or None
        self.username           =   str(data.get('username')).replace("@","")   or None
        self.id                 =   data.get('id')                              or None

        self.messages_parsed    =   data.get('messages_parsed')                 or []
        self.messages_sent      =   data.get('messages_sent')                   or []
        self.messages_received  =   data.get('messages_received')               or []
        self.messages           =   data.get('messages')                        or []

        self.sent_files         =   data.get('sent_files')                      or []
        self.isFavorite         =   data.get('isFavorite')                      or False
        # self.lists              =   data.get('lists')                           or []
        self.start_date         =   data.get('started')                         or None

        # BUG: fix empty array
        if len(self.sent_files) > 0 and self.sent_files[0] == "":
            self.sent_files = []

    def toJSON(self):
        """
        Dumps relevant user data to JSON.
        """

        return json.dumps({
            "name":str(self.name),
            "username":str(self.username),
            "id":str(self.id),
            "messages_parsed":str(self.messages_parsed),
            "messages_sent":str(self.messages_sent),
            "messages_received":str(self.messages_received),
            "messages":str(self.messages),
            "sent_files":str(self.sent_files),
            "isFavorite":str(self.isFavorite)
        })

    def equals(self, user):
        """
        Equals comparison checks usernames and ids.

        Parameters
        ----------
        classes.User
            The user to compare another user object against
        """

        if (str(user.username) != "None" and str(user.username) == str(self.username)) or (str(user.id) != "None" and str(user.id) == str(self.id)): return True
        return False

    def get_id(self):
        """
        Get the provided ID of the User. Searches via username if necessary.

        Returns
        -------
        str
            The user id
        """

        if self.id: return self.id
        self.id = Driver.user_get_id(self.get_username())
        return self.id

    def get_username(self):
        """
        Get the username of the User.

        Returns
        -------
        str
            The username
        """

        if self.username: return self.username
        self.username = Driver.get_username(self.get_id())
        return self.username

    def message(self, message):
        """
        Message the user by their available username or id with the provided message.

        Parameters
        ----------
        message : Object
            The message to send as a serialized Message object from get_message.

        Returns
        -------
        bool
            Whether or not the message was successful
        """

        if not self.get_username() and not self.get_id(): return Settings.err_print("missing user identifiers!")
        if self.id:
            Settings.print("messaging user (id): {} ({}) - \"{}\"".format(self.username, self.id, message["text"]))
        else:
            Settings.print("messaging user: {} - \"{}\"".format(self.username, message["text"]))
        if not Driver.message(self.username, user_id=self.id): return False
        return self.message_send(message)

    def messages_read(self):
        """
        Read the chat of the user.
        """

        Settings.print("reading user chat: {} ({})".format(self.username, self.id))
        # messages, messages_received, messages_sent = Driver.read_user_messages(self.username, user_id=self.id)
        # self.messages = messages
        # self.messages_received = messages_received
        # self.messages_sent = messages_sent
        self.messages, self.messages_received, self.messages_sent = Driver.read_user_messages(self.username, user_id=self.id)
        # self.messages_and_timestamps = messages[1]
        Settings.maybe_print("chat read!")

    def message_send(self, message):
        """
        Complete the various components of sending a message to a user.
        
        Parameters
        ----------
        message : Object
            The message to send as a serialized Message object from get_message.

        Returns
        -------
        bool
            Whether or not the message was successful
        """

        Settings.print("entering message: (${}) {}".format(message["price"], message["text"]))
        try:
            driver = Driver.get_driver()
            def confirm_message(): return driver.message_confirm()
            # enter the text of the message
            def enter_text(text): return driver.message_text(text)
            # enter the price to send the message to the user
            def enter_price(price):
                if not price: return True
                return driver.message_price(price)
            def enter_files(files):
                # TODO: requires proper debugging
                # for file in files:
                    # enter files by filepath while checking for already sent files
                    # file_name = file.get_title()
                    # if str(file_name) in self.sent_files:
                    #     Settings.warn_print("file already sent to user: {} <-- {}".format(self.username, file_name))
                    #     Settings.maybe_print("skipping...")
                    #     continue
                    # self.sent_files.append(file_name)
                return driver.upload_files(files)
            if all([enter_text(message["text"]), enter_price(message["price"]), enter_files(message["files"])]): return confirm_message()
        except Exception as e:
            Settings.err_print("message failed!")
            Settings.dev_print(e)
        Settings.err_print("message somehow failed!")
        return False

    def update(self, user):
        for key, value in json.loads(user.toJSON()).items():
            # Settings.print("updating: {} = {}".format(key, value))
            setattr(self, str(key), value)

    #############
    ## Statics ##
    #############

    # TODO: update with more accurate "active"ness
    # gets users from local or refreshes from onlyfans.com
    @staticmethod
    def get_active_users():
        """
        Get active users.

        Returns
        -------
        list
            The active users

        """

        Settings.dev_print("getting active users...")
        active_users = []
        for user in User.get_all_users():
            if not User.skipUserCheck(user): continue
            active_users.append(user)
        Settings.maybe_print("active users: {}".format(len(active_users)))
        return active_users
    
    @staticmethod
    def get_all_users():
        """
        Get all users.

        Returns
        -------
        list
            The users

        """

        Settings.dev_print("getting all users...")
        users = []
        if Settings.is_prefer_local():
            users = User.read_users_local()
        if len(users) == 0:
            for user in Driver.users_get():
                if user is None: continue
                users.append(User(user))
        Settings.maybe_print("users: {}".format(len(users)))
        User.write_users_local(users=users)
        Settings.set_prefer_local(True)
        return users

    ## TODO
    # make this actually do something
    @staticmethod
    def get_favorite_users():
        """
        Get all favorite users.

        Returns
        -------
        list
            The favorite users

        """

        Settings.dev_print("getting favorite users...")
        users = []
        for user in User.get_all_users():
            if user.isFavorite:
                Settings.maybe_print("fav user: {}".format(user.username))
                users.append(user)
        return users

    @staticmethod
    def get_following():
        """
        Get all following.

        Returns
        -------
        list
            The users being followed

        """

        Settings.dev_print("getting following...")
        if Settings.is_prefer_local():
            users = User.read_following_local()
            if len(users) > 0: return users
        users = []
        for user in Driver.following_get():
            user = User(user)
            users.append(user)
        Settings.maybe_print("following: {}".format(len(users)))
        User.write_following_local(users=users)
        Settings.set_prefer_local(True)
        return users

    @staticmethod
    def get_never_messaged_users():
        """
        Get all users that have never been messaged before.

        Returns
        -------
        list
            The users that have not been messaged

        """

        Settings.dev_print("getting users that have never been messaged...")
        users = []
        for user in User.get_all_users():
            if len(user.messages_received) == 0:
                Settings.maybe_print("never messaged user: {}".format(user.username))
                users.append(user)
        return users

    @staticmethod
    def get_new_users():
        """
        Get all new users.

        Returns
        -------
        list
            The users that are new

        """

        Settings.dev_print("getting new users...")
        newUsers = []
        date_ = datetime.today() - timedelta(days=10)
        for user in User.get_all_users():
            if not user.start_date: continue
            started = datetime.strptime(str(user.start_date),"%b %d, %Y")
            # Settings.maybe_print("date: "+str(date_)+" - "+str(started))
            if started < date_: continue
            Settings.maybe_print("new user: {}".format(user.username))
            newUsers.append(user)
        return newUsers

    @staticmethod
    def get_random_user():
        """
        Get a random user.

        Returns
        -------
        classes.User
            A random user

        """

        Settings.dev_print("getting random user...")

        users = User.get_all_users_usernames()

        randomUser = None
        randomizedUsers = User.get_already_randomized_users()

        while randomUser not in randomizedUsers:
            randomUser = random.choice(users)
            if randomUser not in randomizedUsers:
                User.add_to_randomized_users(randomUser, users=randomizedUsers)
                randomizedUsers.append(randomUser)

        Settings.dev_print("random user: {}".format(randomUser))

        users = User.get_all_users()
        for user in users:
            if str(user.username) == str(randomUser):
                return user
        return User({"username":randomUser})

    @staticmethod
    def get_all_users_usernames():
        users = User.get_all_users()
        usernames = []
        for user in users:
            usernames.append(user.username)
        return usernames

    # return from json file 
    @staticmethod
    def get_already_randomized_users():
        Settings.dev_print("getting already randomized users...")
        users = []
        try:
            with open(str(Settings.get_users_path().replace("users.json","random_users.json"))) as json_file:  
                for user in json.load(json_file)['randomized_users']:
                    users.append(user)
            Settings.maybe_print("loaded randomized users")
        except Exception as e:
            Settings.dev_print(e)
        return users

    # add to json file
    @staticmethod
    def add_to_randomized_users(newUser, users=[]):
        data = {}
        data['randomized_users'] = []
        for user in users:
            data['randomized_users'].append(user)
        data['randomized_users'].append(newUser)
        try:
            with open(str(Settings.get_users_path().replace("users.json","random_users.json")), 'w') as outfile:  
                json.dump(data, outfile, indent=4, sort_keys=True)
        except FileNotFoundError:
            Settings.err_print("missing random users!")
        except OSError:
            Settings.err_print("missing random users path!")

    @staticmethod
    def get_recent_messagers():
        """
        Get users that have recently sent messages.

        Returns
        -------
        list
            The users that have recently sent messages

        """
        Settings.dev_print("getting recent users from messages...")
        users = []
        for user in Driver.messages_scan():
            users.append(User({"id":user}))
        return users

    ## TODO: maybe update this so it actually works?
    @staticmethod
    def get_recent_users():
        """
        Get recent users.

        Returns
        -------
        list
            The recent users

        """
        Settings.dev_print("getting recent users...")
        i = 0
        users = []
        for user in User.get_all_users():
            Settings.maybe_print("recent user: {}".format(user.username))
            users.append(user)
            i += 1
            if i == int(Settings.get_recent_user_count()): break
        return users

    @staticmethod
    def get_user_by_id(userid):
        """
        Get user by id.

        Returns
        -------
        int
            The user id

        """
        if not userid or userid == None:
            Settings.err_print("missing user id")
            return None
        for user in User.get_all_users():
            if str(user.id) == "@u"+str(userid) or str(user.id) == "@"+str(userid) or str(user.id) == str(userid):
                Settings.maybe_print("found user id: {}".format(userid))
                return user
        Settings.err_print("missing user by user id - {}".format(userid))
        return None

    @staticmethod
    def get_user_by_username(username):
        """
        Get user by username.

        Returns
        -------
        classes.User
            The user with the provided username

        """
        if not username or str(username) == "None":
            Settings.err_print("missing username!")
            return None
        for user in User.get_all_users():
            if str(user.username) == "@u"+str(username) or str(user.username) == "@"+str(username) or str(user.username) == str(username):
                Settings.maybe_print("found username: {}".format(username))
                return user
        Settings.err_print("missing user by username - {}".format(username))
        return None

    @staticmethod
    def get_users_by_list(number=None, name=None, ):
        """
        Get users by custom list.

        Returns
        -------
        list
            The users on the list

        """
        Settings.maybe_print("getting users by list: {} - {}".format(number, name))
        listUsers = []
        for user in Driver.get_list(number=number, name=name):
            Settings.maybe_print("user: {}".format(user.username))
            listUsers.append(user)
        return listUsers

    @staticmethod
    def message_user(message, username, user_id=None):

        """
        Message the user by their available username or id with the provided message data.

        Parameters
        ----------
        message : Object
            The message to send as a serialized Message object from get_message.
        """

        if str(username).lower() == "random":
            return User.get_random_user().message(message)
        else:
            return User({"username":username,"id":user_id}).message(message)

    @staticmethod
    def read_following_local():
        """
        Read the locally saved following file.

        Returns
        -------
        list
            The locally saved followers

        """
        Settings.dev_print("getting local following...")
        users = []
        try:
            with open(str(Settings.get_users_path().replace("users.json", "following.json"))) as json_file:  
                for user in json.load(json_file)['users']:
                    users.append(User(json.loads(user)))
            Settings.maybe_print("loaded local following")
        except Exception as e:
            Settings.dev_print(e)
        return users

    @staticmethod
    def read_users_local():
        """
        Read the locally saved users file.

        Returns
        -------
        list
            The locally saved users

        """
        Settings.dev_print("getting local users...")
        users = []
        try:
            with open(str(Settings.get_users_path())) as json_file:  
                for user in json.load(json_file)['users']:
                    users.append(User(json.loads(user)))
            Settings.maybe_print("loaded local users")
        except Exception as e:
            Settings.dev_print(e)
        return users

    @staticmethod
    def read_users_messages(users=[]):
        """
        Read all the users messages.

        Parameters
        ----------
        classes.User
            A list of users to read the messages of.

        """

        if len(users) == 0: users = User.get_all_users()
        Settings.print("updating chat logs: {}".format(len(users)))
        for user in users: user.messages_read()
        # User.write_users_local(users=users)
        return users
 
    @staticmethod
    def skipUserCheck(user):
        """
        Skip user if meets flags.

        Returns
        -------
        classes.User
            The same user provided (if not skipped)

        """
        if str(user.id).lower() in Settings.get_skipped_users() or str(user.username).lower() in Settings.get_skipped_users():
            Settings.maybe_print("skipping: {}".format(user.username))
            return None
        return user

    @staticmethod
    def write_users_local(users=None):
        """
        Write to local users file.

        """
        if users is None:
            users = User.get_all_users()
        if len(users) == 0:
            Settings.maybe_print("skipping: local users save - empty")
            return
        Settings.print("saving users...")
        Settings.dev_print("local users path: "+str(Settings.get_users_path()))
        # merge with existing user data
        existingUsers = User.read_users_local()
        for user in users:
            for u in existingUsers:
                if user.equals(u):
                    user.update(u)
        data = {}
        data['users'] = []
        for user in users:
            data['users'].append(user.toJSON())
        try:
            with open(str(Settings.get_users_path()), 'w') as outfile:  
                json.dump(data, outfile, indent=4, sort_keys=True)
        except FileNotFoundError:
            Settings.err_print("missing local users!")
        except OSError:
            Settings.err_print("missing local path!")

    @staticmethod
    def write_following_local(users=None):
        """
        Write to local followers file.

        """
        if users is None:
            users = User.get_following()
        if len(users) == 0:
            Settings.maybe_print("skipping: local following save - empty following")
            return
        Settings.print("saving following...")
        Settings.dev_print("local users path: "+str(Settings.get_users_path().replace("users.json", "following.json")))
        data = {}
        data['users'] = []
        for user in users:
            data['users'].append(user.toJSON())
        try:
            with open(str(Settings.get_users_path().replace("users.json", "following.json")), 'w') as outfile:  
                json.dump(data, outfile, indent=4, sort_keys=True)
        except FileNotFoundError:
            Settings.err_print("missing local following")
        except OSError:
            Settings.err_print("missing local path")


# parse config file while maintaining default values from args

import configparser
import os

## SAME as Settings.get_base_directory ##
import getpass
USER = getpass.getuser()
# USER = os.getenv('USER')
if str(os.getenv('SUDO_USER')) != "root" and str(os.getenv('SUDO_USER')) != "None":
    USER = os.getenv('SUDO_USER')
configFile = "config.conf"

if not os.environ.get('ENV') or os.environ.get('ENV') == "test":
  configFile = os.path.join(os.getcwd(), "OnlySnarf/conf", "test-config.conf")
elif os.path.isfile(os.path.join("/home/{}/.onlysnarf/conf".format(USER), "config.conf")):
  configFile = os.path.join("/home/{}/.onlysnarf/conf".format(USER), "config.conf")
else:
  configFile = os.path.join(os.getcwd(), "OnlySnarf/conf", "config.conf")

config_file = configparser.ConfigParser()
config_file.read(configFile)

configs = {}

# relabels config for cleaner usage
for section in config_file.sections():
  for key in config_file[section]:
    if section == "ARGS":
      configs[key] = config_file[section][key]
      # print(key, config[key])
    else:
      configs[section.lower()+"_"+key.lower()] = config_file[section][key].strip("\"")
      # print(key, config[section.lower()+"_"+key.lower()])

config = {}

# continue to overwrite values from config file with args
# print(args.items())
# print(os.environ.get('ENV'))
if str(os.environ.get('ENV')) != "test":
  from .args import args
  for key, value in args.items():
    config[key] = value
for key, value in configs.items():
  config[key] = value  

###############
## Debugging ##
# import sys
# print(config)
# sys.exit(0)


import sys
import os

def main(args=None):
    """The main routine."""
    if args is None:
        args = sys.argv[1:]
    os.system("python "+os.path.join(os.path.dirname(os.path.realpath(__file__)),'snarf.py')+" "+" ".join(args))

if __name__ == "__main__":
    try:
        main()
    except:
        print(sys.exc_info()[0])
        print("Shnarf!")
    finally:
        sys.exit(0)


from ..lib.driver import Driver
from ..util.settings import Settings
from .user import User
##
from ..util.defaults import DISCOUNT_MAX_AMOUNT, DISCOUNT_MIN_AMOUNT, DISCOUNT_MAX_MONTHS, DISCOUNT_MIN_MONTHS

class Discount:

    """OnlyFans discount class"""

    def __init__(self, username, amount=None, months=None):

        """OnlyFans discount action."""

        self.amount = amount
        self.months = months
        self.username = username # the recipient username

    def apply(self):

        """
        Applies the discounted amount to the recipient username via Driver.discount_user

        If the targeted username is one of the matching keywords then all of the 
        matching recipients will be discounted. Values are determined by runtime args or prompted
        for.

        """

        Settings.maybe_print("discounting: {}".format(self.username))
        return Driver.discount_user(self.get())

    def get(self):
        """
        Get the discount's values in a dict.

        Returns
        -------
        dict
            A dict containing the values of the discount

        """

        return dict({
            "amount": self.get_amount(),
            "months": self.get_months(),
            "username": self.get_username()
        })

    def get_amount(self):

        """
        Populate and get the amount value

        If not found in args and prompt is enabled, ask for value.

        Returns
        -------
        int
            the discounted amount to apply

        """

        amount = self.amount or Settings.get_amount()
        if int(amount) > int(Settings.get_discount_max_amount()):
            Settings.warn_print("discount amount too high, max -> {}%".format(Settings.get_discount_max_months()))
            amount = int(Settings.get_discount_max_amount())
        elif int(amount) < int(Settings.get_discount_min_amount()):
            Settings.warn_print("discount amount too low, min -> {}%".format(Settings.get_discount_min_months()))
            amount = int(Settings.get_discount_min_amount())
        self.amount = amount
        return self.amount

    def get_months(self):

        """
        Populate and get the months value

        If not found in args and prompt is enabled, ask for value.

        Returns
        -------
        int
            the number of months to discount for

        """

        months = self.months or Settings.get_months()
        # check variable constraints
        if int(months) > int(Settings.get_discount_max_months()):
            Settings.warn_print("discount months too high, max -> {} months".format(Settings.get_discount_max_months()))
            months = int(Settings.get_discount_max_months())
        elif int(months) < int(Settings.get_discount_min_months()):
            Settings.warn_print("discount months too low, min -> {} months".format(Settings.get_discount_min_months()))
            months = int(Settings.get_discount_min_months())
        self.months = months
        return self.months

    def get_username(self):

        """
        Populate and get the username value

        If not found in args and prompt is enabled, ask for value.

        Returns
        -------
        str
            the username to discount

        """

        # if self.username: return self.username
        # self.username = Settings.get_user().username
        return self.username

    def grandfatherer(self, users=[]):

        """
        Executes the 'Grandfather' discount model

        If users is empty it is populated with users from the 'Grandfather' OnlyFans list in 
        the account. All 'Grandfather'ed users are provided with the max discount for the max months.

        Parameters
        ----------
        users : list
            list of users to 'Grandfather'

        """

        if len(users) == 0:
            users = User.get_users_by_list(name="grandfathered")
        print("Discount - Grandfathering: {} users".format(len(users)))
        self.months = DISCOUNT_MAX_MONTHS
        self.amount = DISCOUNT_MAX_AMOUNT
        # apply discount to all users
        for user in users:
            self.username = user.username
            print("Grandfathering: {}".format(self.username))
            try:
                Driver.get_driver().discount_user(discount=self)
            except Exception as e:
                print(e)


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="OnlySnarf",
    version="4.5.8",
    author="Skeetzo",
    author_email="WebmasterSkeetzo@gmail.com",
    url = 'https://github.com/skeetzo/onlysnarf',
    keywords = ['OnlyFans', 'OnlySnarf', 'selenium', 'snarf'],
    description="OnlyFans Content Distribution Tool",
    long_description=long_description,
    long_description_content_type="text/markdown",
    # packages=setuptools.find_packages(),
    packages=["OnlySnarf", "OnlySnarf/classes","OnlySnarf/conf","OnlySnarf/elements","OnlySnarf/lib","OnlySnarf/util"],
    include_package_data=True,
    install_requires=[
        'ffmpeg',
        'inquirer',
        'wget',
        'selenium',
        'webdriver_manager',
        'validators',
        'flask'
        ],
    extras_require={
        'dev': [
            'pytest',
            'flask-unittest'
        ]
    },
    entry_points={
        'console_scripts' : [
            'onlysnarf = OnlySnarf.menu:main',
            'snarf = OnlySnarf.snarf:main',
            'snarfapi = OnlySnarf.api:main'
        ]
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: End Users/Desktop',
        'Topic :: System :: Emulators',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.8',
        "Operating System :: OS Independent"
    ]
)